import JRTCMINIAPP from '../../utils/jrtc-wx'
import md5 from '../../utils/md5'
import TokenUtil from '../../utils/token-util.min'
import { config } from '../../config';
import { preventRepeatClick } from '../../utils/rtcUtil';

Page({
  /**
   * 页面的初始数据
   */
  data: {
    broadcaster: [],
    audience: [],
    broadcasterCount: 0,
    audienceCount: 0,
    total: 0,
    roleType: 2,
    publishUrl: '',
    pullUrl: '',
    isMe: false,
    isLeaveRoom: false,
    muteUserId: '',
    cancelMuteUserId: '',
    isMuted: false,
    isAudio: false,
    userRoomId: null,
    roomId: null,  // JRTC roomId, 由用户的appId和userRoomId生成的房间Id
    triggerSetRole: false,
    isClick: true,
    globalMuteLoading: false,
    globalMuteCancelLoading: false
  },

// 播放器组件state变化监听函数  
statechangePlayer(e) {
  console.log('live-player e', e)
  if (e.detail.code === 6001) {
    this.pullerCtx.play({
      success: res => {
        console.log('播放流成功:', res)
      },
      fail: err => {
        console.log('播放流失败:', err)
      }
    })
  }
},  

// 播放器组件网络状态监听函数
netstatusPlayer(e) {
  console.log('live-player -netstatusPlaye- e', e)
},

errorHandle(e) {
  console.log('errorHandle code:', e)
},

// 推流组件状态变化监听
statechange(e) {
  console.log('live-pusher e:', e)
  console.log('this.pusherCtx-', this.pusherCtx)
  if((e.detail.message && e.detail.message.includes('stop service')) || e.detail.code === 2029) {
    this.pusherCtx.start({
     success: res => {
       console.log('启动推流成功-', res)
     },
     fail: err => {
       console.log('启动推流失败', err)
     }
    })
    this.pusherCtx.setBGMVolume({
      volume: 1,
      success: res => {
        console.log('设置背景音量成功:', res)
      },
      fail: err => {
        console.log('设置背景音量失败', err)
      }
    })
 }
}, 

async initRoom(userRoomId) {
  const { appId, appKey } = config
  this.setData({
    userRoomId
  })
  const initClientData = {
    appId,
    // 测试环境推拉流域名-gray
    // publishDomain: '116.198.2.22:1937/live-jrtc-test.jdcloud.com',
    // pullDomain: '116.198.2.22:1936/live-jrtc-test.jdcloud.com'
    // 线上推拉流域名
    publishDomain: 'live-jrtc.push.jdcloud.com',
    pullDomain: 'live-jrtc.play.jdcloud.com'
  }

  const initInstance = await JRTCMINIAPP.init(initClientData)

  console.log('initInstance:', initInstance)
  if(initInstance.code === 431) {
    wx.showToast({
      title: initInstance.reason + '-errorCode: ' + initInstance.code,
      icon: 'none'
    })
    setTimeout(() => {
      wx.reLaunch({
        url: '/pages/meetingEnter/meetingEnter',
      })
    }, 2000)
    return;
  }

  const nickName = wx.getStorageSync('myWxUseInfo').nickName
  const userId = md5(nickName);
  const nonce = 'AK-' + userId;
  const timestamp = new Date().getTime();
  const token = TokenUtil.hmacSha256(appId, appKey, userId,userRoomId,nonce,timestamp);

  const enterData = {
    appId,
    userRoomId,
    nickName,
    userId,
    role: 2,
    nonce,
    timestamp,
    token
  }

   const roomObj = await JRTCMINIAPP.joinRoom(enterData);
   const { code, data } = roomObj;
   if(code !== 200 ) {
      wx.showToast({
        title: roomObj.reason,
        duration: 2000
      })
   }
  // 本地存储自身信息
   const myInfo = Object.assign({}, {
    nickName, 
    userRoomId,
    userId
   })
  console.log('myInfo***', myInfo)
  wx.setStorageSync('myInfo', myInfo)
   
   let _pullUrl = ''
   // 创建live-player对象
   this.pullerCtx = wx.createLivePlayerContext('puller');
   // 创建live-pusher对象
   this.pusherCtx = wx.createLivePusherContext('livePusher');
   if (code === 200) {
     if (data.role === 2) {
       _pullUrl = await JRTCMINIAPP.subscribe();
       console.log('观众，获取pullUrl:', _pullUrl)
     }
     this.setData({
      pullUrl: _pullUrl,
      roleType: data.role,
      roomId: data.roomId
    })
    console.log('----data---', this.data)
    this.getRoomMemberCount();
    this.getAudienceList();
    this.getBroadcasterList();
   } else {
     wx.showToast({
       title: roomObj.reason,
       duration: 1000,
       icon: 'success'
     })
     setTimeout(() => {
      wx.navigateBack({
        delta: 0,
      })
     }, 1500)
   }
},

// 离开房间
async leaveRoom() {
  this.setData({
    isLeaveRoom: true
  })
  console.log('leaveRoom-isLeaveRoom:', this.data.isLeaveRoom)
  const leaveRoom = await JRTCMINIAPP.leaveRoom();
  console.log('leaveRoom-result:', leaveRoom);
  const { code } = leaveRoom;
  if(code === 200) {
    wx.showToast({
      title: '成功离开',
      duration: 2000,
      icon: 'success'
    });
    wx.reLaunch({
      url: '/pages/meetingEnter/meetingEnter',
    })
  }
},

setRole() {
  // 防抖
  preventRepeatClick(this, setRoleHandle, 800, 'isClick')
  async function setRoleHandle() {
     const _roleType = this.data.roleType === 1 ? 2 : 1;
     const changeRole = await JRTCMINIAPP.setRole(_roleType);
     const { code, data } = changeRole;
     if (code === 200) {
       const role = data.role;
       wx.setStorageSync('myRole', role)
       console.log('current role--', role)
       if (role*1 === 1 || !role) {
          this.setData({
            isMe: true,
            roleType: role,
            publishUrl: data.publishAudioUrl,
            pullUrl: data.audioUrl
          })
       } else {
          this.setData({
            isMe: false,
            roleType: role,
            publishUrl: '',
            pullUrl: data.audioUrl
          })
       }
     }
  }
},

// 发送至房间
async customToRoom() {
    const appData = {
      aa: 11,
      bb: 22
    }
    const unpubishData = await JRTCMINIAPP.customSignalToRoom(appData);
    const { code, data } = unpubishData;
    if(code === 200) {
      wx.showToast({
        title: data,
        duration: 2000,
        icon: 'success'
      })
    }
},

// 发布音频
async publish() {
  const publishUrl = await JRTCMINIAPP.publish();
  this.setData({
    publishUrl
  })
},

// 订阅远端音频流
async subscribe() {
  const pullUrl = await JRTCMINIAPP.subscribe();
  this.setData({
    pullUrl
  })
},

// 全局静音
async globalMute() {
  const appData = {
    customEventName: 'GlobalMute'
  }
  // 防止多次点击按钮
  if(!this.data.globalMuteLoading) {
    await JRTCMINIAPP.customSignalToRoom(appData);
    this.setData({
      globalMuteLoading: true
    })
    setTimeout(()=> {
      this.setData({
        globalMuteLoading: false
      })
    }, 1000)
  }  
},

// 取消全局静音
async cancelGlobalMute() {
  const appData = {
    customEventName: 'CancelGlobalMute'
  }
  if(!this.data.globalMuteCancelLoading) {
    await JRTCMINIAPP.customSignalToRoom(appData);
    this.setData({
      globalMuteCancelLoading: true
    })
    // 防止多次点击按钮
    setTimeout(()=> {
      this.setData({
        globalMuteCancelLoading: false
      })
    }, 1000)
  }
},

// 指定用户静音
async assignPeerMute() {
  console.log('指定某用户静音', this.data.muteUserId)
  const { muteUserId } = this.data
  const appData = {
    targetUserId : muteUserId,
    customData: {
      customEventName: 'AssignPeerMute',
      msg: `你好呀${muteUserId}`
    }
  }
  await JRTCMINIAPP.customSignalToPeer(appData);
  this.setData({
    muteUserId: ''
  })
},

// 取消指定某用户静音
async cancelAssignPeerMute() {
  const cancelMuteUserId = this.data.cancelMuteUserId
  const appData = {
    targetUserId : cancelMuteUserId,
    customData: {
      customEventName: 'CancelAssignPeerMute',
      msg: `取消${cancelMuteUserId}静音`
    }
  }
  await JRTCMINIAPP.customSignalToPeer(appData);
  this.setData({
    cancelMuteUserId: ''
  })
},

// 获取房间成员数
async getRoomMemberCount() {
  const roomMembers = await JRTCMINIAPP.getRoomMemberCount();
  const { code, data } = roomMembers;
  if (code === 200) {
    console.log('打印房间人数：', data)
    const { total, broadcasterCount, audienceCount } = data;
    this.setData({
      total,
      broadcasterCount,
      audienceCount
    })
  }
},

// 获取主播列表
async getBroadcasterList() {
    const broadcasterList = await JRTCMINIAPP.getBroadcasterList();
    const { code, data } = broadcasterList;
    const _myUserId = wx.getStorageSync('myInfo').userId
    const isExistMe = data.findIndex(item => item.userId === _myUserId) < 0 ? false : true;
    if (code === 200) {
      if (isExistMe) {
        const myInfo = data.filter(item => item.userId === _myUserId)[0]
        const exceptMeBroadcasterList = data.filter(item => item.userId !== _myUserId)
        exceptMeBroadcasterList.unshift(myInfo)
        this.setData({
          broadcaster: exceptMeBroadcasterList
        })
      } else {
        this.setData({
          broadcaster: data
         });
      }
    }
},

// 获取观众列表
async getAudienceList() {
     const audienceList = await JRTCMINIAPP.getAudienceList();
     const { code, data } = audienceList;
     const _myUserId = wx.getStorageSync('myInfo').userId
     const isExistMe = data.findIndex(item => item.userId === _myUserId) < 0 ? false : true;
     if (code === 200) {
       if (isExistMe) {
        const myInfo = data.filter(item => item.userId === _myUserId)[0]
        const exceptMeAudienceList = data.filter(item => item.userId !== _myUserId)
        exceptMeAudienceList.unshift(myInfo)
        this.setData({
          audience: exceptMeAudienceList
         })
       } else {
        this.setData({
          audience: data
         })
       }
     }
},

// 消息监听处理函数
receiveMsg(data, type) {
  console.log('接收到数据：', data, type)
  const _myUserId = wx.getStorageSync('myInfo').userId
  const { userId, nickName, code } = data
  switch (type) {
      case 'onError':
         const { reason } = data
         switch(code) {
          case 431: {
            wx.showToast({
              title: reason + '-errorCode: ' + code,
              icon: 'none'
            })
            setTimeout(() => {
              JRTCMINIAPP.destroy();
              wx.reLaunch({
                url: '/pages/meetingEnter/meetingEnter',
              })
            }, 2000)
            break;
          }
          case 503: {
            wx.showToast({
              title: reason + '-errorCode: ' + code,
              icon: 'none'
            })
            break;
         }
           case 1003: {
              wx.showToast({
                title: reason + '-errorCode: ' + code,
                icon: 'none'
              })
              break;
           }
           case 1006: {
            wx.showToast({
              title: reason + '-errorCode: ' + code,
              icon: 'none'
            })
            setTimeout(() => {
              JRTCMINIAPP.destroy();
              wx.reLaunch({
                url: '/pages/meetingEnter/meetingEnter',
              })
            }, 2000)
            break;
           }
           case 1007: {
            if(this.data.isLeaveRoom) return;
            wx.showToast({
              title: reason + '-errorCode: ' + code,
              icon: 'none'
            })
            setTimeout(() => {
              JRTCMINIAPP.destroy();
              wx.reLaunch({
                url: '/pages/meetingEnter/meetingEnter',
              })
            }, 2000)
            break;
           }
           default:
             console.log(data, type)
         }
         break
      case 'UserJoinedRoom':
          wx.showToast({
            title: '用户【' + nickName + '】加入房间',
            icon: 'none'
          })
          this.handleUserJoinedRoom(data)
          break
      case 'UserModifyRole':
        this.handleUserModifyRole(data)
        break    
      case 'UserLeavedRoom':
          wx.showToast({
            title: '用户【' + nickName + '】离开房间',
            icon: 'none'
          })
          this.handleUserLeave(data)
          break
      case 'StreamPublished':
          this.handleStreamPublished(data)
          break  
      case 'StreamUnpublished':
          wx.showToast({
            title: '用户【' + nickName + '】停止推流',
            icon: 'none'
          })
          const { broadcaster } = this.data;
          if(typeof data.isAudio === "boolean") {
            const newBroadcasterList = broadcaster.map(item => {
              if(item.userId === userId) {
                item.isAudio = data.isAudio
              }
              return item
            })
            this.setData({
              broadcaster: newBroadcasterList
            })
          }
          this.handleUserLeave(data)
          break       
      case 'GlobalMute':
        if(_myUserId !== userId) {
          this.handleMute(data);
        }
        break
      case 'CancelGlobalMute':
        this.handleCancelMute(data);
        break       
      case 'AssignPeerMute':
          this.handleAssignPeerMute(data);
          break  
      case 'CancelAssignPeerMute':
        this.handleCancelAssignPeerMute(data)
        break 
      case 'AudioVolumes':
        this.handleAudioVolumes(data)
        break     
      default:
          console.log(type, data)
  }
},

handleUserJoinedRoom(data) {
  const { role, userId } = data;
  console.log('handleUserJoinedRoom:', data)
  const { broadcaster, audience } = this.data;
  // 判断用户是否已存在
  const isExistBroadcaster = broadcaster.some(item => item.userId === userId)
  const isExistAudience = audience.some(item => item.userId === userId)

  if(isExistBroadcaster || isExistAudience) {
    wx.showToast({
      title: '该用户已存在',
      duration: 2000,
      icon: 'none'
    })
    this.getRoomMemberCount()
    this.getBroadcasterList()
    this.getAudienceList()
    return
  }

  if (role*1 === 1 || !role) {
    const _broadcaster = broadcaster.concat(data);
    const _obj = {}
    const _broadcasterDeDup = _broadcaster.reduce((prev, cur) => {
      _obj[cur.userId] ? '' : _obj[cur.userId] = true && prev.push(cur)
      return prev
    }, [])
    this.setData({
      broadcaster: _broadcasterDeDup,
      broadcasterCount: _broadcasterDeDup.length,
      total: _broadcasterDeDup.length + audience.length
    })
  } else {
    const _audience = audience.concat(data);
    const _obj = {}
    const _audienceDeDup = _audience.reduce((prev, cur) => {
      _obj[cur.userId] ? '' : _obj[cur.userId] = true && prev.push(cur)
      return prev
    }, [])
    this.setData({
      audience: _audienceDeDup,
      audienceCount: _audienceDeDup.length,
      total: _audienceDeDup.length + broadcaster.length
    })
  }
},

handleUserLeave(data) {
  const { userId } = data;
  const { broadcaster, audience } = this.data;
  const role = audience.findIndex(item => item.userId === userId) < 0 ? 1 : 2;
  if (role*1 === 1 || !role) {
    const _broadcasterRole1 = broadcaster.filter(item => item.userId !== userId)
    this.setData({
      broadcaster: _broadcasterRole1,
      broadcasterCount: _broadcasterRole1.length,
      total: _broadcasterRole1.length + audience.length
    })
  } else {
    const _audienceRole2 = audience.filter(item => item.userId !== userId)
    this.setData({
      audience: _audienceRole2,
      audienceCount: _audienceRole2.length,
      total: _audienceRole2.length + broadcaster.length
    })
  }
},

async handleUserModifyRole(data) {
  const { userId, nickName, role } = data;
  const { broadcaster, audience } = this.data;
  const _myUserId = wx.getStorageSync('myInfo').userId
  if(role*1 === 1 || !role) {
    wx.showToast({
      title: '用户【' + nickName + '】成为主播',
      icon: 'none'
    })
    const _broadcasterRole1 = broadcaster.concat(audience.filter(item => item.userId === userId))
    const _audienceRole1 = audience.filter(item => item.userId !== userId)
    let _myInfo = {}
    let _sortBroadcaster = []
    _broadcasterRole1.map(item => {
      if(item.userId === _myUserId) {
        _myInfo = item
      } else {
        _sortBroadcaster.push(item)
      }
    })
    if(JSON.stringify(_myInfo) !== '{}') {
      _sortBroadcaster.unshift(_myInfo)
    }
    this.setData({
      audience: _audienceRole1,
      broadcaster: _sortBroadcaster,
      broadcasterCount: _sortBroadcaster.length,
      audienceCount: _audienceRole1.length
    })
  } else {
    const _broadcasterRole2 = broadcaster.filter(item => item.userId !== userId)
    const _audienceRole2 = audience.concat(broadcaster.filter(item => item.userId === userId))
    let _myInfo = {}
    let _sortAudience = []
    _audienceRole2.map(item => {
       if(item.userId === userId && item.userId === _myUserId) {
        _myInfo = item
       } else {
        _sortAudience.push(item)
       }
     })
     if(JSON.stringify(_myInfo) !== '{}') {
       _sortAudience.unshift(_myInfo)
     }
    this.setData({
      broadcaster: _broadcasterRole2,
      audience: _sortAudience,
      broadcasterCount: _broadcasterRole2.length,
      audienceCount: _sortAudience.length
    })
  }
},

handleMute(data) {
  const { nickName } = data
  const { isMuted } = this.data
  wx.showToast({
    title: '被嘉宾【' + nickName + '】设置为全局静音',
    icon: 'none'
  })
  if(isMuted) return;
  this.pullerCtx.mute({
    success: res => {
      this.setData({
        isMuted: true
      })
    },
    fail: res => {
      console.log('mute fail')
    }
  })
},

handleAssignPeerMute(data) {
  console.log('handleAssignPeerMute信令返回参数:', data)
  const { nickName } = data
  const { isMuted } = this.data
  wx.showToast({
    title: '被嘉宾【' + nickName + '】设置为静音',
    icon: 'none'
  })
  if(isMuted) return;
  this.pullerCtx.mute({
    success: res => {
      this.setData({
        isMuted: true
      })
    },
    fail: res => {
      console.log('mute fail')
    }
  })
},

handleCancelAssignPeerMute(data) {
  console.log('信令返回参数:', data)
  const { nickName } = data
  const { isMuted } = this.data
  wx.showToast({
    title: '被嘉宾【' + nickName + '】取消静音',
    icon: 'none'
  })
  if(!isMuted) return;
  this.pullerCtx.mute({
    success: res => {
      this.setData({
        isMuted: false
      })
      console.log('handleCancelMute success')
    },
    fail: res => {
      console.log('handleCancelMute fail')
    }
  })
},

handleCancelMute(data) {
  console.log('信令返回参数:', data)
  const { nickName } = data
  const { isMuted } = this.data
  wx.showToast({
    title: '被嘉宾【' + nickName + '】取消全局静音',
    icon: 'none'
  })
  if(!isMuted) return;
  this.pullerCtx.mute({
    success: res => {
      this.setData({
        isMuted: false
      })
    },
    fail: res => {
      console.log('handleCancelMute fail')
    }
  })
},

handleStreamPublished(data) {
  const { broadcaster } = this.data
  if (data.length > 1) {
    console.log('多条数据')
  } else if(data.length === 1) {
    const _currentBroadcaster = data[0]
    const newBroadcasterList = broadcaster.map(item => {
      if(item.userId === _currentBroadcaster.userId) {
        item.hasAudio = _currentBroadcaster.hasAudio
      }
      return item
    })
    this.setData({
      broadcaster: newBroadcasterList
    })
  }
},

handleAudioVolumes(data) {
  const { broadcaster } = this.data
  const _hasAudioVolumeBroadcasters = data
    const newBroadcasterList = broadcaster.map(item => {
      _hasAudioVolumeBroadcasters.map(hasAudioVolume => {
        if(item.userId === hasAudioVolume.userId) {
          item.isAudio = true
        }
      })
      return item
    })
    this.setData({
      broadcaster: newBroadcasterList
    })
},

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: async function (options) {
    const { userRoomId } = options;
    JRTCMINIAPP.onMsg((data, type) => {
      this.receiveMsg(data, type)
    })
    await this.initRoom(userRoomId);
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: async function (options) {
    
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {
  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {
    if(JRTCMINIAPP.JRTCMiniAppRoom) {
      JRTCMINIAPP.destroy();
    }
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  }
})