import JRTCMiniappClient from '../lib/jrtc-miniapp.min'

let jrtcMiniapp= null

class JRTCMINIAPP {
  constructor() {
    this.onMsgCallbacks = []
    this.msgs = []
    this.isInit = false
    this.JRTCMiniAppRoom = null
  }

  isInit() {
    return this.isInit
  }
 
  // sdk实例初始化
  async init(initData) {
    if (jrtcMiniapp) return jrtcMiniapp;
    jrtcMiniapp = await JRTCMiniappClient.init(initData);
    if (jrtcMiniapp.code === 431) {
      return jrtcMiniapp
    }
    this.errorListener();
    this.isInit = true
    return jrtcMiniapp
  }
 
  // 进入房间
  joinRoom(enterInfo) {
    const that = this;
    return new Promise((resolve, reject) => {
      jrtcMiniapp.joinRoom(enterInfo).then(res => {
         that.JRTCMiniAppRoom = res;
         that.messageListener();
         resolve(res)
      }).catch(err => {
         reject(err)
      })
    })
  }

  // 设置角色
  setRole(roleType) {
    return new Promise((resolve, reject) => {
      jrtcMiniapp.setRole(roleType).then(res => {
        resolve(res)
      }).catch(err => {
        reject(err)
      })
    })
  }

  // 获取主播列表
  getBroadcasterList() {
    return new Promise((resolve, reject) => {
      jrtcMiniapp.getBroadcasterList().then(res => {
        resolve(res)
      }).catch(err => {
        reject(err)
      })
    })
  }

  // 获取观众列表
  getAudienceList() {
    return new Promise((resolve, reject) => {
      jrtcMiniapp.getAudienceList().then(res => {
        resolve(res)
      }).catch(err => {
        reject(err)
      })
    })
  }
  
  // 由用户业务逻辑决定，如全局静音
  customSignalToRoom(appData) {
    return new Promise((resolve, reject) => {
      jrtcMiniapp.customSignalToRoom(appData).then(res => {
        resolve(res)
      }).catch(err => {
        reject(err)
      })
    })
  }

  // 发送给指定用户信息， 如举手
  customSignalToPeer(appData) {
    return new Promise((resolve, reject) => {
      jrtcMiniapp.customSignalToPeer(appData).then(res => {
        resolve(res)
      }).catch(err => {
        reject(err)
      })
    })
  }

  // 获取成员数
  getRoomMemberCount() {
    return new Promise((resolve, reject) => {
      jrtcMiniapp.getRoomMemberCount().then(res => {
        resolve(res)
      }).catch(err => {
        reject(err)
      })
    })
  }

  // 退出房间
  leaveRoom() {
    return new Promise((resolve, reject) => {
      jrtcMiniapp.leaveRoom().then(res => {
        jrtcMiniapp.destroy()
        resolve(res)
      }).catch(err => {
        reject(err)
      })
    })
  }

  // 发布音频
  publish() {
    return new Promise((resolve, reject) => {
      jrtcMiniapp.publish().then(res => {
        resolve(res)
      }).catch(err => {
        reject(err)
      })
    })
  }

  // 订阅远端音频流
  subscribe() {
    return new Promise((resolve, reject) => {
      jrtcMiniapp.subscribe().then(res => {
        resolve(res)
      }).catch(err => {
        reject(err)
      })
    })
  }

  // 销毁客户端对象
  destroy() {
    const that = this;
    that.clearMsg();
    return jrtcMiniapp.destroy();
  }

  // 设置错误监听
  errorListener() {
    const that = this;
    jrtcMiniapp.on('onError', (err) => {
      that.onMsgHandle(err, 'onError');
    })
  }

   // room消息监听
   messageListener() {
    const that = this;
    jrtcMiniapp.on('UserJoinedRoom', (data) => {
      console.log('用户进入-消息监听：', data)
      that.onMsgHandle(data, 'UserJoinedRoom');
    });

    jrtcMiniapp.on('UserLeavedRoom', (data) => {
      console.log('用户离开-消息监听：', data)
      that.onMsgHandle(data, 'UserLeavedRoom');
    });

    jrtcMiniapp.on('UserModifyRole', (data) => {
      console.log('用户角色改变-消息监听', data)
      that.onMsgHandle(data, 'UserModifyRole');
    });

    jrtcMiniapp.on('GlobalMute', (data) => {
      console.log('全局静音-消息监听', data)
      that.onMsgHandle(data, 'GlobalMute');
    });

    jrtcMiniapp.on('CancelGlobalMute', (data) => {
      console.log('取消全局静音-消息监听', data)
      that.onMsgHandle(data, 'CancelGlobalMute');
    });

    jrtcMiniapp.on('AssignPeerMute', (data) => {
      console.log('指定用户静音-消息监听', data)
      that.onMsgHandle(data, 'AssignPeerMute');
    });

    jrtcMiniapp.on('CancelAssignPeerMute', (data) => {
      console.log('取消指定用户静音-消息监听', data)
      that.onMsgHandle(data, 'CancelAssignPeerMute');
    });

    jrtcMiniapp.on('StreamPublished', (data) => {
      console.log('推流-消息监听', data)
      that.onMsgHandle(data, 'StreamPublished');
    });

    jrtcMiniapp.on('StreamUnpublished', (data) => {
      console.log('取消推流-消息监听', data)
      that.onMsgHandle(data, 'StreamUnpublished');
    });

    jrtcMiniapp.on('AudioVolumes', (data) => {
      console.log('音量变化-消息监听', data)
      that.onMsgHandle(data, 'AudioVolumes');
    });

    jrtcMiniapp.on('onError', (err) => {
      console.log('错误-消息监听', err)
      console.log(err)
    })
  }

  /**
   * onMsgHandle 消息函数处理
   * @param {*} emitData 自定义事件emit 传递的数据
   * @param {*} eventType 自定义事件
   */
  onMsgHandle(emitData, eventType) {
    let that = this;
    that.msgs.push({ emitData, eventType });
    that.onMsgCallbacks.forEach(fn => {
      fn(emitData, eventType);
    });
  }
  
  // 消息监听函数
  onMsg(callback) {
    const that = this;
    that.onMsgCallbacks.push(callback);
    if (that.msgs.length) {
      that.msgs.forEach((item) => {
        callback(item.msg, item.type);
      });
    }
  }

  clearMsg() {
    const that = this;
    that.onMsgCallbacks = []
    that.msgs = []
  }
}
export default new JRTCMINIAPP(); 